@@ -1,29 +1,22 @@
-/*
- * ssheet.c -- definition of the languages style sheets
- *
- * Copyright (c) 1988-93 Miguel Santana
- * Copyright (c) 1995-99 Akim Demaille, Miguel Santana
- *
- */
+/* ssheet.c - definition of the languages style sheets
+   Copyright 1988-2017 Free Software Foundation, Inc.
 
-/*
- * This file is part of a2ps.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
+
+#include <config.h>
 
 #include "a2ps.h"
 #include "routines.h"
@@ -52,16 +45,16 @@ extern char * re_syntax_table;
 
 /* This function is defined in sshparser.y, but I don't know
  * how to make it appear in a .h file */
-struct style_sheet * parse_style_sheet PARAMS ((const char * filename));
+struct style_sheet * parse_style_sheet (const char * filename);
 
 /* Computes the sum of the versions and the highest requirement
  * of the sheet with key in KEYS */
-static inline void style_sheets_versions PARAMS ((struct darray * keys,
+static inline void style_sheets_versions (struct darray * keys,
 						  int sum [4],
-						  int requirement [4]));
+						  int requirement [4]);
 
 /* return true iff OK */
-static bool style_sheet_check PARAMS ((struct style_sheet * sheet));
+static bool style_sheet_check (struct style_sheet * sheet);
 
 /* We drop in the alphabets[0] an invalid value.  It allows to see if
  * the alphabets were defined or not, so that the default alphabets
@@ -74,13 +67,13 @@ static bool style_sheet_check PARAMS ((struct style_sheet * sheet));
 #define ALPHABET_UNDEFINE(_alpha_)	\
 	((_alpha_)[0] = ALPHABET_UNDEFINED_TAG)
 
-#define DEFAULT_ALPHA1 ((const uchar *) \
+#define DEFAULT_ALPHA1 ((const unsigned char *) \
 	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_")
-#define DEFAULT_ALPHA2 ((const uchar *) \
+#define DEFAULT_ALPHA2 ((const unsigned char *) \
 	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789")
 
 static void
-alphabet_self_print (char *a, FILE *s)
+alphabet_self_print (unsigned char *a, FILE *s)
 {
   if (ALPHABET_IS_UNDEFINED (a))
     fputs ("<undefined>\n", s);
@@ -97,11 +90,11 @@ alphabet_self_print (char *a, FILE *s)
 }
 
 #define DEFAULT_SENSITIVENESS case_insensitive
-
+
 struct pattern *
 new_pattern (char * pattern, size_t len)
 {
-  struct pattern * res = XMALLOC (struct pattern, 1);
+  struct pattern * res = XMALLOC (struct pattern);
   res->pattern = pattern;
   res->len = len;
   return res;
@@ -111,9 +104,9 @@ new_pattern (char * pattern, size_t len)
  * The faced_string
  */
 struct faced_string *
-faced_string_new (uchar * string, int reg_ref, struct fface_s face)
+faced_string_new (char * string, int reg_ref, struct fface_s face)
 {
-  struct faced_string * res = XMALLOC (struct faced_string, 1);
+  struct faced_string * res = XMALLOC (struct faced_string);
   res->string = string;
   res->reg_ref = reg_ref;
   res->face = face;
@@ -134,13 +127,6 @@ faced_string_self_print (struct faced_string * faced_string, FILE * stream)
   fface_self_print (faced_string->face, stream);
 }
 
-inline static void
-faced_string_free (struct faced_string * faced_string)
-{
-  XFREE (faced_string->string);
-  free (faced_string);
-}
-
 /*
  * Replace all the No_fface's by FACE
  */
@@ -150,7 +136,7 @@ faced_string_set_no_face (struct faced_string * str, struct fface_s face)
   if (fface_squ (str->face, No_fface))
     str->face = face;
 }
-
+
 /************************************************************************/
 /*	The version handling						*/
 /************************************************************************/
@@ -179,7 +165,7 @@ style_sheet_set_requirement (struct style_sheet * sheet,
 
   return true;
 }
-
+
 /************************************************************************/
 /*	Rhs darray						*/
 /************************************************************************/
@@ -199,7 +185,7 @@ rhs_new (void)
  * New rhs array with a single argument
  */
 struct darray *
-rhs_new_single (uchar * string, int reg_ref, struct fface_s face)
+rhs_new_single (char * string, int reg_ref, struct fface_s face)
 {
   struct darray * res;
   res = da_new ("Rhs", 2, da_geometrical, 2,
@@ -246,7 +232,7 @@ rhs_self_print (struct darray * rhs, FILE * stream)
     faced_string_self_print ((struct faced_string *) rhs->content [i], stream);
   putc (')', stream);
 }
-
+
 /************************************************************************/
 /*	P-rules								*/
 /************************************************************************/
@@ -254,10 +240,10 @@ rhs_self_print (struct darray * rhs, FILE * stream)
  * Dealing with the rules and operators
  */
 inline static struct rule *
-rule_new_internal_word (uchar * word,
+rule_new_internal_word (char * word,
 			struct darray * rhs)
 {
-  struct rule * res = XMALLOC (struct rule, 1);
+  struct rule * res = XMALLOC (struct rule);
 
   res->word = word;
   res->regex = NULL;
@@ -270,19 +256,19 @@ rule_new_internal_word (uchar * word,
  */
 inline static struct rule *
 rule_new_internal_regexp (struct pattern *pattern,
-			  char *regexp, int regexp_len,
+			  char *regexp, size_t regexp_len,
 			  struct darray *rhs,
 			  const char *filename, size_t line)
 {
-  struct rule *res = XMALLOC (struct rule, 1);
+  struct rule *res = XMALLOC (struct rule);
   const char *error_msg;
 
   /* This is a regular expression.  We want to keep the original
    * pattern to ease the debugging of a style sheet */
-  res->word = (uchar *) pattern->pattern;
+  res->word = pattern->pattern;
 
   /* Build the regex structure, and compile the pattern */
-  res->regex = XMALLOC (struct re_pattern_buffer, 1);
+  res->regex = XMALLOC (struct re_pattern_buffer);
   res->regex->translate = NULL;
   res->regex->fastmap = NULL;
   res->regex->buffer = NULL;
@@ -291,10 +277,9 @@ rule_new_internal_regexp (struct pattern *pattern,
   error_msg = re_compile_pattern (regexp, regexp_len, res->regex);
   /* The pattern is nul terminated, no fear to have */
   if (error_msg)
-    error_at_line (1, 0, filename, line,
+    error_at_line (1, 0, filename, (unsigned) line,
 		   _("cannot compile regular expression `%s': %s"),
 		   regexp, error_msg);
-  free (pattern);
   res->rhs = rhs;
   return res;
 }
@@ -303,7 +288,7 @@ rule_new_internal_regexp (struct pattern *pattern,
  * Dealing with the rules and operators
  */
 struct rule *
-rule_new (uchar * word, struct pattern * pattern,
+rule_new (char * word, struct pattern * pattern,
 	  struct darray * rhs,
 	  const char *filename, size_t line)
 {
@@ -327,11 +312,11 @@ rule_new (uchar * word, struct pattern * pattern,
 `-------------------------------------------------------------------*/
 
 struct rule *
-keyword_rule_new (uchar * word, struct pattern * pattern,
+keyword_rule_new (char * word, struct pattern * pattern,
 		  struct darray * rhs,
 		  const char *filename, size_t line)
 {
-  int pattern_len = 0;
+  size_t pattern_len = 0;
   char * pattern_to_compile = NULL;
 
   if (pattern)
@@ -350,23 +335,6 @@ keyword_rule_new (uchar * word, struct pattern * pattern,
 }
 
 
-/*--------------.
-| Free a rule.  |
-`--------------*/
-
-static void
-free_rule (struct rule * rule)
-{
-  XFREE (rule->word);
-  if (rule->regex)
-    {
-      regfree (rule->regex);
-      free (rule->regex);
-    }
-  da_free (rule->rhs, (da_map_func_t) faced_string_free);
-  free (rule);
-}
-
 /*
  * Compare two rules.
  * Note: they should be string-rules, not regex-rule
@@ -374,7 +342,7 @@ free_rule (struct rule * rule)
 static int
 rule_cmp (struct rule * k1, struct rule * k2)
 {
-  return ustrcmp (k1->word, k2->word);
+  return strcmp (k1->word, k2->word);
 }
 
 static void
@@ -406,7 +374,7 @@ rules_set_no_faces (struct darray * arr, struct fface_s face)
     rule_set_no_face (((struct rule *) arr->content[i]), face);
 }
 
-
+
 /*
  * `words' are a specialization of darray, to capture the fact
  * that it is faster to look for a prefix of a string taking
@@ -420,7 +388,7 @@ words_new (/* Regular darray parameters */
 	      size_t size, size_t increment)
 {
   int c;
-  struct words * res = XMALLOC (struct words, 1);
+  struct words * res = XMALLOC (struct words);
 
   /* Initialize the underlying darray of strings */
   res->strings = da_new (name_strings, size,
@@ -441,24 +409,6 @@ words_new (/* Regular darray parameters */
   return res;
 }
 
-static inline void
-words_erase (struct words * words)
-{
-  /* Do not free the items, there may be pointers onto them */
-  da_erase (words->strings);
-  da_erase (words->regexps);
-
-  free (words);
-}
-
-void
-words_free (struct words * words)
-{
-  da_free (words->strings, (da_map_func_t) free_rule);
-  da_free (words->regexps, (da_map_func_t) free_rule);
-  free (words);
-}
-
 static void
 words_self_print (struct words * words, FILE * stream)
 {
@@ -502,12 +452,7 @@ words_merge_rules_unique (struct words * words, struct words * new)
 
   /* Include the strings */
   da_qsort (new->strings);
-  da_merge (words->strings, new->strings,
-	    (da_map_func_t) free_rule, da_2_wins);
-
-  /* Empty the structure, but don't free, since words->regexps
-   * keeps a reference to them */
-  words_erase (new);
+  da_merge (words->strings, new->strings, da_2_wins);
 }
 
 /*
@@ -526,12 +471,12 @@ words_finalize (struct words * list)
    * and last occurence index in (*DEST) array */
   content = (struct rule **) list->strings->content;
   for (i = 0 ; i < list->strings->len ; i++) {
-    if (list->min [content[i]->word[0]] == NULL)
-      list->min [content[i]->word[0]] = content + i;
-    list->max [content[i]->word[0]] = content + i;
+    if (list->min [(unsigned char) content[i]->word[0]] == NULL)
+      list->min [(unsigned char) content[i]->word[0]] = content + i;
+    list->max [(unsigned char) content[i]->word[0]] = content + i;
   }
 }
-
+
 /************************************************************************/
 /*		Handling the ancestors of a style sheet 		*/
 /************************************************************************/
@@ -563,13 +508,12 @@ ancestors_finalize (struct style_sheet * sheet)
 
       /* Inherit from their string keywords.  The keywords we already
 	 have are the oldest, hence the one to keep.  This is why
-	 da_1_wins.  Do not free the item, coz' your being killing
-	 another style sheet! */
+	 da_1_wins. */
       da_merge (sheet->keywords->strings, ancestor->keywords->strings,
-		NULL, da_1_wins);
+		da_1_wins);
       /* Inherit from their string operators */
       da_merge (sheet->operators->strings, ancestor->operators->strings,
-		NULL, da_1_wins);
+		da_1_wins);
     }
 
   /* We want to inherit from the _last_ alphabets if it has not been
@@ -588,9 +532,9 @@ ancestors_finalize (struct style_sheet * sheet)
       /* Inherit of the _last_ alphabets if it has not been defined in
        * this sheet.  */
       if (ALPHABET_IS_UNDEFINED (sheet->alpha1))
-	memcpy (sheet->alpha1, ancestor->alpha1, sizeof (uchar) * 256);
+	memcpy (sheet->alpha1, ancestor->alpha1, sizeof (unsigned char) * 256);
       if (ALPHABET_IS_UNDEFINED (sheet->alpha2))
-	memcpy (sheet->alpha2, ancestor->alpha2, sizeof (uchar) * 256);
+	memcpy (sheet->alpha2, ancestor->alpha2, sizeof (unsigned char) * 256);
 
       /* Inherit of the _last_ case sensitivity */
       if (sheet->sensitiveness == case_undefined)
@@ -609,7 +553,7 @@ ancestors_finalize (struct style_sheet * sheet)
       da_prefix (sheet->sequences, ancestor->sequences);
     }
 }
-
+
 /************************************************************************/
 /*		Creating, loading a style sheet				*/
 /************************************************************************/
@@ -622,7 +566,7 @@ sequence_new (struct rule * Open,
 	      struct words * Close,
 	      struct words * exceptions)
 {
-  struct sequence * res = XMALLOC (struct sequence, 1);
+  struct sequence * res = XMALLOC (struct sequence);
 
   /* Make sure to set the face of those with No_fface */
   rule_set_no_face (Open, in_face);
@@ -640,25 +584,13 @@ sequence_new (struct rule * Open,
   return res;
 }
 
-/*
- * Release a sequence.
- */
-void
-free_sequence (struct sequence * sequence)
-{
-  free_rule (sequence->open);
-  words_free (sequence->close);
-  words_free (sequence->exceptions);
-  free (sequence);
-}
-
 /*
  * Build the usual C's string and char with their escapes
  */
 #define C_add_exception(_excep_)					\
   words_add_string (res,						\
-	     rule_new (xustrdup (_excep_), NULL,			\
-		       rhs_new_single (xustrdup (_excep_), 0, String_fface),\
+	     rule_new (xstrdup (_excep_), NULL,			\
+		       rhs_new_single (xstrdup (_excep_), 0, String_fface),\
                        __FILE__, __LINE__));
 
 static struct words *
@@ -680,15 +612,15 @@ new_C_exceptions (void)
 struct sequence *
 new_C_string_sequence (const char * delimitor)
 {
-  struct sequence * res = XMALLOC (struct sequence, 1);
-  res->open = rule_new (xustrdup (delimitor), NULL,
+  struct sequence * res = XMALLOC (struct sequence);
+  res->open = rule_new (xstrdup (delimitor), NULL,
 			rhs_new_single (NULL, 0, Plain_fface),
 			__FILE__, __LINE__);
   res->face = String_fface;
   res->exceptions = new_C_exceptions ();
   res->close = words_new ("C Close: strings", "C Close: regex", 5, 5);
   words_add_string (res->close,
-		    rule_new (xustrdup (delimitor), NULL,
+		    rule_new (xstrdup (delimitor), NULL,
 			      rhs_new_single (NULL, 0, Plain_fface),
 			      __FILE__, __LINE__));
   words_finalize (res->close);
@@ -705,7 +637,7 @@ sequence_self_print (struct sequence * tmp, FILE * stream)
   words_self_print (tmp->exceptions, stream);
   words_self_print (tmp->close, stream);
 }
-
+
 /************************************************************************/
 /*		Creating, loading a style sheet				*/
 /************************************************************************/
@@ -713,14 +645,14 @@ sequence_self_print (struct sequence * tmp, FILE * stream)
  * Dealing with the style sheets
  */
 struct style_sheet *
-new_style_sheet (const uchar * name)
+new_style_sheet (const char * name)
 {
-  NEW (struct style_sheet, res);
+  struct style_sheet * res = XMALLOC (struct style_sheet);
 
   res->name = name;
-  res->author = UNULL;
+  res->author = NULL;
   version_set_to_null (res->version);
-  res->documentation = UNULL;
+  res->documentation = NULL;
   version_set_to_null (res->requirement);
   res->ancestors = ancestors_new ();
   res->sensitiveness = case_undefined;
@@ -746,7 +678,7 @@ new_style_sheet (const uchar * name)
 `---------------------------------------------------------*/
 
 static char *
-style_sheet_mixed_new (const uchar * ancestors)
+style_sheet_mixed_new (const char * ancestors)
 {
   struct style_sheet *sheet, *ancestor;
   char *ancestor_key, *key, *cp;
@@ -754,7 +686,7 @@ style_sheet_mixed_new (const uchar * ancestors)
   struct darray *ancestors_array;
 
   /* It cannot be longer than ancestors_keys. */
-  key = ALLOCA (char, ustrlen (ancestors) + 1);
+  key = ALLOCA (char, strlen (ancestors) + 1);
   astrcpy (ancestors_keys, ancestors);
 
   /* Create the darray of ancestors keys, and build the final key. */
@@ -778,10 +710,9 @@ style_sheet_mixed_new (const uchar * ancestors)
   message (msg_sheet,
 	   (stderr, "Creating a mixed style sheet \"%s\"\n", key));
   /* Its name is its key. */
-  sheet = new_style_sheet ((uchar *) key);
+  sheet = new_style_sheet (key);
   sheet->key = strdup (key);
   da_concat (sheet->ancestors, ancestors_array);
-  da_erase (ancestors_array);
   style_sheet_finalize (sheet);
 
   return key;
@@ -851,7 +782,7 @@ load_style_sheet (const char * pseudo_key)
       /* This style sheet has to have its path as key.  This is used
 	 to ease to allow specifying a style sheet by its real path,
 	 instead of allowing key based file search.  It is also used
-	 to to check that an ssh file is OK, from a2ps-mode in
+	 to check that an ssh file is OK, from a2ps-mode in
 	 emacs.*/
       res = parse_style_sheet (pseudo_key);
     }
@@ -867,7 +798,6 @@ load_style_sheet (const char * pseudo_key)
 	  return NULL;
 	}
       res = parse_style_sheet (path);
-      free (path);
     }
 
   /* Now, finalize it, and store it in the hash tab */
@@ -876,7 +806,7 @@ load_style_sheet (const char * pseudo_key)
 
   return res;
 }
-
+
 /************************************************************************/
 /*		Reporting information about the style sheets		*/
 /************************************************************************/
@@ -934,12 +864,12 @@ style_sheet_self_print (struct style_sheet * sheet, FILE * stream)
 static void
 style_sheet_print_signature (FILE * stream, struct style_sheet * sheet)
 {
-  int i, title_bar_len;
+  size_t i, title_bar_len;
 
   if (!version_null_p (sheet->version))
     {
       title_bar_len = (strlen (" (.ssh version )")
-		       + ustrlen (sheet->name)
+		       + strlen (sheet->name)
 		       + strlen (sheet->key)
 		       + version_length (sheet->version));
       fprintf (stream, "%s (%s.ssh version ",
@@ -950,7 +880,7 @@ style_sheet_print_signature (FILE * stream, struct style_sheet * sheet)
   else
     {
       title_bar_len = (strlen (" (.ssh)")
-		       + ustrlen (sheet->name)
+		       + strlen (sheet->name)
 		       + strlen (sheet->key));
       fprintf (stream, "%s (%s.ssh)\n",
 	       sheet->name, sheet->key);
@@ -1000,9 +930,8 @@ list_style_sheets_long (FILE * stream)
       sheet = get_style_sheet ((char *) entries->content[i]);
       style_sheet_print_signature (stream, sheet);
     }
-  da_free (entries, (da_map_func_t) free);
 }
-
+
 /************************************************************************/
 /*		Report in HTML format					*/
 /************************************************************************/
@@ -1066,9 +995,8 @@ list_style_sheets_html (FILE * stream)
 {
   struct style_sheet * sheet;
   struct darray * entries;
-  size_t i;
+  size_t i, rows;
   version_t version_index, requirement;
-  int rows;
 
   entries = pw_glob_on_suffix (job->common.path, SSH_SUFFIX);
 
@@ -1160,10 +1088,8 @@ them all.  Its index (sum of the style sheets version number) is \
 #{html.end.hook}\n\
 </body>\n\
 </html>\n"), stream);
-
-  da_free (entries, (da_map_func_t) free);
 }
-
+
 /************************************************************************/
 /*		Report in Texinfo format				*/
 /************************************************************************/
@@ -1249,10 +1175,8 @@ The current index (sum of all the style sheets version number) is ", stream);
     sheet = get_style_sheet ((char *) entries->content[i]);
     style_sheet_texinfo_print_signature (stream, sheet);
   }
-
-  da_free (entries, (da_map_func_t) free);
 }
-
+
 /************************************************************************/
 /*		Perform some tests on the style sheets			*/
 /************************************************************************/
@@ -1352,8 +1276,8 @@ check_rules_doubles (const char * name, struct darray * rules)
   bool res = true;
 
   for (i = 1 ; i < rules->len ; i++)
-    if (!ustrcmp (((struct rule *) rules->content [i - 1])->word,
-		  ((struct rule *) rules->content [i])->word))
+    if (STREQ (((struct rule *) rules->content [i - 1])->word,
+               ((struct rule *) rules->content [i])->word))
       {
 	res = false;
 	error (0, 0, "%s.ssh:`%s' is defined twice",
@@ -1393,7 +1317,7 @@ style_sheet_check (struct style_sheet * sheet)
 {
   return check_doubles (sheet) && check_sensitivity (sheet);
 }
-
+
 /************************************************************************/
 /*			Handling of the hash table of			*/
 /*				the style sheets			*/
@@ -1427,7 +1351,7 @@ struct hash_table_s *
 new_style_sheets (void)
 {
   struct hash_table_s * res;
-  res = XMALLOC (hash_table, 1);
+  res = XMALLOC (hash_table);
   hash_init (res, 8,
              sheet_hash_1, sheet_hash_2, sheet_hash_cmp);
   return res;
@@ -1442,7 +1366,7 @@ require_style_sheet (const char * key)
   struct style_sheet * item;
   struct style_sheet token;
 
-  if (strequ (key, "plain"))
+  if (STREQ (key, "plain"))
     return;
 
   token.key = (char *) key;
@@ -1475,7 +1399,7 @@ get_style_sheet (const char * key)
   struct style_sheet * item;
   struct style_sheet token;
 
-  if (strequ (key, "plain"))
+  if (STREQ (key, "plain"))
     return NULL;
 
   /* There is some specials magic to build on the fly mixed style
@@ -1498,14 +1422,14 @@ get_style_sheet (const char * key)
   if (!item)
     return NULL;
 
-  /* To me, it looks like an horror: the syntax table is not
-   * stored in the regex buffer.  As a consequence, each time
-   * we change sheet, make sure to update the re_syntax_table */
+  /* To me, it's a horror: the syntax table is not stored in the regex
+   * buffer.  As a consequence, each time we change sheet, make sure to
+   * update the re_syntax_table */
   re_syntax_table = (char *) item->alpha1;
 
   return item;
 }
-
+
 /*
  * Compute the total SUM of the all the versions of the
  * sheets which key is in KEYS
